# Generated by Django 3.0.4 on 2020-04-05 01:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("getpaid", "0002_auto_20200403_1554"),
    ]

    operations = [
        migrations.AlterField(
            model_name="payment",
            name="amount_locked",
            field=models.DecimalField(
                decimal_places=2,
                default=0,
                help_text="Amount locked with this payment, ready to charge.",
                max_digits=20,
                verbose_name="amount paid",
            ),
        ),
        migrations.AlterField(
            model_name="payment",
            name="amount_paid",
            field=models.DecimalField(
                decimal_places=2,
                default=0,
                help_text="Amount actually paid.",
                max_digits=20,
                verbose_name="amount paid",
            ),
        ),
        migrations.AlterField(
            model_name="payment",
            name="amount_required",
            field=models.DecimalField(
                decimal_places=2,
                help_text="Amount required to fulfill the payment; in selected currency, normal notation",
                max_digits=20,
                verbose_name="amount required",
            ),
        ),
    ]
